;=============================================================================
; (C) 1999-2001 by Helge Wagner
;
; FILE: RM95EMGY.ASM
;
; DESC: Sony LANC service mode remote control RM95 emulation 
;	(emergency function, will set page 0x02 address 0x25 to 0x80)
;
; BY:   H.Wagner, mailto:<H-Wagner@gmx.net>
; DATE: 22.05.2001
;
; NOTE: Tab width=4
;=============================================================================
; Rev  Date       Author   Description
;-----------------------------------------------------------------------------
;=============================================================================

.MODEL  SMALL,SYSCALL
 PAGE   60,132
 TITLE  RM95EMGY.ASM
 OPTION PROC:private

.nolist
  include lanc_if.inc
.list

;%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
;%
;%                MAIN MODULE
;%
;%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%

; equates for the supported interfaces
IF_IRDEO	EQU 0
IF_PAR1517	EQU 1

PRG_DATA_SEGM SEGMENT PARA PUBLIC 'DATA' USE16

stdout_redir        byte 0          ; 0: stdout is not redirected

user_break          byte 0

current_page        byte ?
page_locked         byte ?          ; 1=address can't be incremented on current page
org 100h
eeprom_page_data    byte 256 dup (?)

PRG_DATA_SEGM ENDS

CODE SEGMENT PARA PUBLIC 'CODE' USE16
.586p

jmp   start

include lanc_ui.asm

start:
  mov   ax, seg PRG_DATA_SEGM
  mov   ds, ax
assume ds:PRG_DATA_SEGM
  mov   al, byte ptr es:[18h+1]
  .if (al != 1)
    mov   stdout_redir, 1
  .endif
  
  call  do_the_job                  ; this could set error code in al
  mov   ah, 4Ch
  int   21h

;in:    al      command, where
;               0   = NOP (used for reading the eeprom)
;               67h = page+
;               65h = page-
;               38h = addr+
;               36h = addr-
;               34h = data+
;               30h = data-
;               32h = store data
;out:   carry set       error
;       carry clear     ok, then
;           es:bx   pointer to last received lanc telegram
rm95_command proc uses ax cx
local send_b1:byte, send_b2:byte

  .if (al == 0) || (al == 67h) || (al == 65h) || (al == 38h) || (al == 36h) || (al == 34h) || (al == 30h) || (al == 32h)
    jmp   ok
  .else
    stc
    ret
  .endif

ok:
  mov   send_b1, 0FFh
  mov   send_b2, al

  .if (send_b2 == 0)
    jmp   read_command
  .endif
  xor   cx, cx
  .repeat
    mov   al, send_b1
    mov   ah, send_b2
    call  lanc_if_send
    call  lanc_if_receive_telegram
    .if (al != NOERROR)
      stc
      ret
    .endif
    mov   al, byte ptr es:[bx+5]
    inc   cx
  .until (al == 0F1h) || (cx > 6)

read_command:
  xor   cx, cx
  .repeat
    mov   al, send_b1
    mov   ah, 0
    call  lanc_if_send
    call  lanc_if_receive_telegram
    .if (al != NOERROR)
      stc
      ret
    .endif
    mov   al, byte ptr es:[bx+5]
    inc   cx
  .until (al == 0F0h) || (cx > 100)

  .if (cx > 100)
    stc
    ret
  .endif

  clc

  ret

rm95_command endp

;in:    ah      page (0-15)
;out:   carry set       error
;       carry clear     ok, then
;           es:bx   pointer to last received lanc telegram
rm95_setpage proc uses ax cx
; new page (np), old page (op)
; pd=np-op
; wenn pd > 0 und pd <= 8 inc page
; wenn pd < 0 oder pd > 8 dec page

local pg:byte, curr_pg:byte
local command:byte

  .if (ah > 15)
    stc
    ret
  .endif

  mov   pg, ah

  mov   al, 0                       ; 0=read eeprom data
  call  rm95_command
  .if carry?
    ret
  .endif

  mov   ah, es:byte ptr [bx+4]
  shr   ah, 4
  mov   curr_pg, ah
  mov   al, pg
  .if (al < ah)
    add   al, 16
  .endif
  sub   al, ah
  .if (al == 0)
    jmp   pg_ok
  .elseif (al <= 8)        ; inc page
    mov   command, 67h
  .else
    mov   ah, al
    mov   al, 16
    sub   al, ah
    mov   command, 65h
  .endif
  movzx cx, al
  .repeat
    mov   al, command
    call  rm95_command
    .if carry?
      ret
    .endif
  .untilcxz

pg_ok:

  ret

rm95_setpage endp


;in:    ah      address (0-255)
;out:   carry set       error
;       carry clear     ok, then
;           es:bx   pointer to last received lanc telegram
rm95_setaddress proc uses ax bx cx
; new address (na), old address (oa)
; pd=na-oa
; wenn pd > 0 und pd <= 128 inc address
; wenn pd < 0 oder pd > 128 dec address

local address:byte
local command:byte

  mov   address, ah

  mov   al, 0                       ; 0=read eeprom data
  call  rm95_command
  .if carry?
    ret
  .endif

  movzx ax, es:byte ptr [bx+6]
  movzx bx, address
  .if (bx < ax)
    add   bx, 256
  .endif
  sub   bx, ax
  .if (bx == 0)
    jmp   addr_ok
  .elseif (bx <= 128)        ; inc address
    mov   command, 38h
  .else
    mov   ax, bx
    mov   bx, 256
    sub   bx, ax
    mov   command, 36h
  .endif
  mov   cx, bx
  .repeat
    mov   al, command
    call  rm95_command
    .if carry?
      ret
    .endif
  .untilcxz

addr_ok:

  ret

rm95_setaddress endp



;in:    ah      data (0-255)
;out:   carry set       error
;       carry clear     ok, then
;           es:bx   pointer to last received lanc telegram
rm95_setdata proc uses ax bx cx
; new data (nd), old data (od)
; pd=nd-od
; wenn pd > 0 und pd <= 128 inc data
; wenn pd < 0 oder pd > 128 dec data

local data:byte
local command:byte

  mov   data, ah

  mov   al, 0                       ; 0=read eeprom data
  call  rm95_command
  .if carry?
    ret
  .endif

  movzx ax, es:byte ptr [bx+7]
  movzx bx, data
  .if (bx < ax)
    add   bx, 256
  .endif
  sub   bx, ax
  .if (bx == 0)
    jmp   data_ok
  .elseif (bx <= 128)        ; inc data
    mov   command, 34h
  .else
    mov   ax, bx
    mov   bx, 256
    sub   bx, ax
    mov   command, 30h
  .endif
  mov   cx, bx
  .repeat
    mov   al, command
    call  rm95_command
    .if carry?
      ret
    .endif
  .untilcxz

data_ok:

  ret

rm95_setdata endp

cannot_set_page_msg	byte 13,10,"Fatal error: can not set page!", 13,10,0
cannot_set_address_msg	byte 13,10,"Fatal error: can not set address!", 13,10,0
cannot_set_data_msg	byte 13,10,"Fatal error: can not set data!", 13,10,0
emergency_ok_msg	byte "All done! Emergency function ok", 13,10,0
page_ok_msg		byte "page:done",0
address_ok_msg		byte ", address:done",0
data_ok_msg		byte ", data:done   :)   ",0

rm95_emergency proc uses ax si

local pg:byte, address:byte, data:byte

  mov   pg, 02h
  mov   address, 25h
  mov   data, 80h

  cli
  call  lanc_if_telegram_syncronize

  mov   ah, pg
  call  rm95_setpage
  .if (carry?)
    call  lanc_if_telegram_syncronize
    call  rm95_setpage
    .if (carry?)
      call  lanc_if_telegram_syncronize
      call  rm95_setpage
      .if (carry?)
        sti
        mov   si, offset cannot_set_page_msg
        call  write_string_tty
        jmp   done
      .endif
    .endif
  .endif
  sti

  mov   si, offset page_ok_msg
  call  write_string_tty
  
  cli
  call  lanc_if_telegram_syncronize
  mov   ah, address
  call  rm95_setaddress
  .if (carry?)
    call  lanc_if_telegram_syncronize
    call  rm95_setaddress
    .if (carry?)
      call  lanc_if_telegram_syncronize
      call  rm95_setaddress
      .if (carry?)
        sti
        mov   si, offset cannot_set_address_msg
        call  write_string_tty
        jmp   done
      .endif
    .endif
  .endif
  sti

  mov   si, offset address_ok_msg
  call  write_string_tty
  
  cli
  call  lanc_if_telegram_syncronize
  mov   ah, data
  call  rm95_setdata
  .if (carry?)
    call  lanc_if_telegram_syncronize
    call  rm95_setdata
    .if (carry?)
      call  lanc_if_telegram_syncronize
      call  rm95_setdata
      .if (carry?)
        sti
        mov   si, offset cannot_set_data_msg
        call  write_string_tty
        jmp   done
      .endif
    .endif
  .endif
  sti

  mov   si, offset data_ok_msg
  call  write_string_tty
  
  mov   si, offset emergency_ok_msg
  call  write_string_tty

done:

  ret

rm95_emergency endp

;in	none
;out	al:interface, either IF_IRDEO or IF_PAR1517
get_interface proc
local interface:byte

 ; get the interface information:
 ;  irdeo: lanc_if_set_channel with channel = 2 will work, channel=3 will fail
  mov   interface, IF_IRDEO

  mov   al, 2
  call  lanc_if_set_channel
  .if (al != NOERROR)
   ; can't be IRdeo
    mov   interface, IF_PAR1517
    jmp   interface_known
  .endif
  mov   al, 3
  call  lanc_if_set_channel
  .if (al == E__LANC_WRONG_CHANNEL)
   ; must be IRdeo
    mov   interface, IF_IRDEO
    jmp   interface_known
  .endif
  mov   interface, IF_PAR1517

interface_known:

 ; restore the default LANC channel
  mov   al, 0
  call  lanc_if_set_channel

  mov   al, interface
  
  ret

get_interface endp


welcome_msg         byte "RM95EMGY Release 1.0, (C) 1999-2001 by H.Wagner",13,10,0
interface_par1517_msg   byte "Used interface: PAR1517",13,10,0
interface_irdeo_msg     byte "Used interface: IRdeo",13,10,0
init_lanc_msg       byte "Initializing LANC_IF library, please wait...",0
calibrating_msg     byte "Now calibrating, please wait...",0
done_msg            byte 8,8," Done.", 13,10,0
press_anykey_msg    byte "press any key to continue...", 0
start_camcorder_msg byte "Please start your camcorder in Player mode now...", 13,10
                    byte "(press ESC to break)", 0
lanc_not_found_msg  byte 13,10,"Lanc device not found. Stopped.", 13,10,0
lanc_found_msg      byte 13,10,"OK. Lanc device found.", 13,10,0
break_accepted_msg  byte 13,10,"User break accepted.", 13,10,0
error1_msg          byte "Error1", 13,10,0
error2_msg          byte "Error2", 13,10,0
error3_msg          byte "Error3", 13,10,0
error4_msg          byte "Error4", 13,10,0
error5_msg          byte "Error5", 13,10,0
error6_msg          byte "Error6", 13,10,0

;=============================================================================
;FUNC:  DO_THE_JOB
;
;DESC:  Do whatever you want
;
;IN:    NONE
;OUT:   AL      error level
;NOTE:
;=============================================================================
do_the_job proc near

  mov   si, offset welcome_msg
  call  write_string_tty

  mov   si, offset init_lanc_msg
  call  write_string_tty
  call  lanc_if_init
  mov   si, offset done_msg
  call  write_string_tty

  call  lanc_if_open
  
  call  get_interface
  .if (al == IF_IRDEO)
    mov   si, offset interface_irdeo_msg
  .else
    mov   si, offset interface_par1517_msg
  .endif
  call  write_string_tty

  mov   si, offset start_camcorder_msg
  call  write_string_tty
  .repeat
    call  check_break
    .break .if (carry?)
    call  lanc_if_check_active
  .until (bl == 0)
  .if (carry?)
    mov   si, offset break_accepted_msg
    call  write_string_tty
    ret
  .endif
  mov   si, offset crlf_msg
  call  write_string_tty

  mov   si, offset calibrating_msg
  call  write_string_tty
  call  lanc_if_calibrate
  .if (al != NOERROR)
    mov   si, offset error3_msg
    call  write_string_tty
    ret
  .endif
  mov   si, offset done_msg
  call  write_string_tty

  call rm95_emergency

  ret

do_the_job endp

CODE ENDS

END start
